<?php
class Query {
    /**
     * Element
     *
     * @var DOMElement
     */
    private $_elem;
    /**
     * Document
     *
     * @var DOMDocument
     */
    private $_dom;
    /**
     * Constructor
     *
     * @param DOMDocument $dom
     */
    public function __construct($dom) {
        $this->_dom = $dom;
    }
    /**
     * get id
     *
     * @param  string $id
     * @return Query
     */
    public function id($id = null) {
        if ($id === null) {
            $ts = debug_backtrace(false);
            foreach ($ts as $t) {
                if ($t['class'] !== get_class($this)) {
                    $id = $t['function'];
                    break;
                }
            }
        }
        $this->_elem = $this->_dom->getElementById($id);
        if ($this->_elem === null) {
            throw new Exception('id is not found.');
        }
        return $this;
    }
    /**
     * set and get Attribute
     *
     * @param  string|array $key
     * @param  string $value
     * @return string|Query
     */
    public function attr($key = null, $value = null) {
        $this->_hasId();
        if ($value === null) {
            if (is_string($key)) {
                return $this->_elem->getAttribute($key);
            } elseif (is_array($key)) {
                foreach ($key as $k => $v) {
                    $this->_elem->setAttribute((string)$k, (string)$v);
                }
                return $this;
            }
        } else {
            $this->_elem->setAttribute((string)$key, (string)$value);
            return $this;
        }
    }
    /**
     * removeAttribute
     *
     * @param  string $name
     * @return Query
     */
    public function removeAttr($name = null) {
        if ($name === null) throw new Exception('$name is null');
        $this->_hasId();
        $this->_elem->removeAttribute($name);
        return $this;
    }
    /**
     * add class
     *
     * @param  string $class
     * @return Query
     */
    public function addClass($class = null) {
        if ($class === null) throw new Exception('$class is null');
        $this->_hasId();
        $clazz = $this->attr('class');
        if ($clazz === null) {
            $clazz .= ' ';
        }
        $clazz .= $class;
        $this->attr('class', $clazz);
        return $this;
    }
    /**
     * remove class
     *
     * @param  string $class
     * @return Query
     */
    public function removeClass($class = null) {
        if ($class === null) throw new Exception('$class is null');
        $this->_hasId();
        $clazz = explode(' ', $this->attr('class'));
        $a = array();
        foreach ($clazz as $c) {
            if (empty($c) || strcmp($class, $c) === 0) {
                continue;
            }
            $a[] = $c;
        }
        $this->attr('class', implode(' ', $a));
        return $this;
    }
    /**
     * add or remove class
     *
     * @param  string $class
     * @return Query
     */
    public function toggleClass($class = null) {
        if ($class === null) throw new Exception('$class is null');
        $this->_hasId();
        $clazz = explode(' ', $this->attr('class'));
        foreach ($clazz as $c) {
            if (strcmp($c, $class) === 0) {
                return $this->removeClass($class);
            }
        }
        return $this->addClass($class);
    }
    /**
     * set or get html
     *
     * @param  string $val
     * @return string|Query
     */
    public function html($val = null) {
        $this->_hasId();
        if ($val === null) {
            $nodes = $this->_elem->childNodes;
            $txt = '';
            foreach ($nodes as $node) {
                $name = $node->nodeName;
                $value = $node->nodeValue;
                if ($name !== '#text') {
                    $txt .= '<' . $name;
                    if ($node->attributes !== null) {
                        foreach ($node->attributes as $k) {
                            $txt .= ' ' . $k->name . '="' . $k->value . '"';
                        }
                    }
                    $txt .= '>';
                    $txt .= $value;
                    $txt .= '</' . $name . '>';
                } else {
                    $txt .= $value;
                }
            }
            return $txt;
        } else {
            $this->clean()->append($val);
        }
        return $this;
    }
    /**
     * set or get value
     *
     * @param  string $val
     * @return string|Query
     */
    public function val($val = null) {
        $this->_hasId();
        if ($val === null) {
            return $this->_elem->getAttribute('value');
        } else {
            $this->_elem->setAttribute('value', (string)$val);
        }
        return $this;
    }
    /**
     * get or set text
     *
     * @param  string $val
     * @return string|Query
     */
    public function text($val = null) {
        $this->_hasId();
        if ($val === null) {
            return $this->_elem->textContent;
        } else {
            $this->clean()->appendText($val);
            return $this;
        }
    }
    /**
     * clean child element
     *
     * @return Query
     */
    public function clean() {
        $this->_hasId();
        while($this->_elem->hasChildNodes()) {
            $nodes = $this->_elem->childNodes;
            foreach($nodes as $node) {
                $this->_elem->removeChild($node);
            }
        }
        return $this;
    }
    /**
     * append string
     *
     * @param  string $str
     * @return Query
     */
    public function append($str) {
        $this->_hasId();
        $newdoc = new DOMDocument();
        $newdoc->loadHTML($str);
        $body = $newdoc->getElementsByTagName('body')->item(0);
        $nodes = $body->childNodes;
        foreach ($nodes as $node) {
            $e = $this->_dom->importNode($node, true);
            $this->_elem->appendChild($e);
        }
        return $this;
    }
    /**
     * append text
     *
     * @param  string $str
     * @return Query
     */
    public function appendText($str) {
        $this->_hasId();
        $doc = $this->_dom;
        $e = $doc->createTextNode($str);
        $this->_elem->appendChild($e);
        return $this;
    }
    /**
     * getDomDocument
     *
     * @return DOMDocument
     */
    public function getDom() {
        return $this->_dom;
    }
    /**
     * loop
     * 
     * @param array $obj
     */
    public function loop(array $obj) {
        $count = count($obj);
        for ($i = 1; $i <= $count; $i++) {
            $this->_hasId();
            $title = $this->_elem->getAttribute('title');
            if (array_key_exists($title, $obj[$i - 1])) {
                $val = is_array($obj[$i - 1]) ? (object)$obj[$i - 1]->$tile : $obj[$i - 1]->$title;
                $this->text($val);
                $c = clone $this->_elem;
                $this->_elem->removeAttribute('id');
                if ($i !== $count) {
                    $this->_elem->parentNode->appendChild($c);
                    $this->_elem = $c;
                }
            } else {
                $nodes = $this->_elem->childNodes;
                $flg = false;
                foreach ($nodes as $node) {
                    if ($node->nodeType === XML_ELEMENT_NODE) {
                        $title = $node->getAttribute('title');
                        if (array_key_exists($title, $obj[$i - 1])) {
                            $val = is_array($obj[$i - 1]) ? (object)$obj[$i - 1]->$tile : $obj[$i - 1]->$title;
                            $cs = $node->childNodes;
                            foreach ($cs as $c) {
                                $node->removeChild($c);
                            }
                            $text = $this->_dom->createTextNode($val);
                            $node->appendChild($text);
                            $flg = true;
                        }
                    }
                }
                if (!$flg) {
                    throw new Exception('target not found.');
                }
                $c = clone $this->_elem;
                $this->_elem->removeAttribute('id');
                if ($i !== $count) {
                    $this->_elem->parentNode->appendChild($c);
                    $this->_elem = $c;
                }
            }
        }
    }
    /**
     * has id
     * @return void
     */
    private function _hasId() {
        if ($this->_elem === null) {
            $this->id();
            return;
        }
        $name = $this->_elem->getAttribute('id');
        $ts = debug_backtrace(false);
        foreach ($ts as $t) {
            if ($t['class'] !== get_class($this)) {
                $id = $t['function'];
                break;
            }
        }
        if ($name === $id) return;
        $this->id($id);
        return;
    }
}